@@ -73,6 +73,8 @@ import time
 import traceback as traceback_
 import logging
 import platform
+import contextlib
+import threading
 
 try:
     from functools import lru_cache
@@ -1157,6 +1159,12 @@ class HTTPRequest:
                     # Closing the conn is the only way to determine len.
                     self.close_connection = True
 
+        # Override the decision to not close the connection if the connection
+        # manager doesn't have space for it.
+        if not self.close_connection:
+            can_keep = self.server.connections.can_add_keepalive_connection
+            self.close_connection = not can_keep
+
         if b'connection' not in hkeys:
             if self.response_protocol == 'HTTP/1.1':
                 # Both server and client are HTTP/1.1 or better
@@ -1801,6 +1809,18 @@ class HTTPServer:
         self.prepare()
         self.serve()
 
+    @contextlib.contextmanager
+    def _run_in_thread(self):
+        """Context manager for running this server in a thread."""
+        self.prepare()
+        thread = threading.Thread(target=self.serve)
+        thread.setDaemon(True)
+        thread.start()
+        try:
+            yield thread
+        finally:
+            self.stop()
+
     def error_log(self, msg='', level=20, traceback=False):
         """Write error message to log.
 
